(* $Id: godi_digest.ml 799 2008-06-03 15:13:33Z gerd $ 
 * ----------------------------------------------------------------------
 * Copyright 2008 by Gerd Stolpmann.
 *
 * This file is part of godi-tools.
 *
 * godi-tools is free software; you can redistribute it and/or modify it
 * under the conditions in the file LICENSE.
 *
 * godi-tools is distributed WITHOUT ANY WARRANTY, as detailed in the
 * file LICENSE.
 *)

type digest_kind = [ `MD5 | `SHA1 ]


module MD5 = struct
  let digest_of_string = Digest.string
  let digest_of_channel c = Digest.channel c (-1)
end


module SHA1 = struct
  external sha1_init : unit -> unit = "sha1_init"
  external sha1_update : string -> int -> unit = "sha1_update"
  external sha1_final : unit -> string = "sha1_final"

  let digest_of_obj_channel ch =
    sha1_init();
    let s = String.create 4096 in
    ( try
	while true do
	  let n = ch # input s 0 4096 in
	  sha1_update s n
	done;
	assert false
      with
	| End_of_file -> ()
    );
    sha1_final()

  let digest_of_string s =
    (* sha1_update does not like long strings, so use an obj channel *)
    let ch = new Netchannels.input_string s in
    digest_of_obj_channel ch

  let digest_of_channel c =
    digest_of_obj_channel (new Netchannels.input_channel c)

end


let digest_of_string =
  function
    | `MD5 -> MD5.digest_of_string
    | `SHA1 -> SHA1.digest_of_string


let digest_of_channel =
  function
    | `MD5 -> MD5.digest_of_channel
    | `SHA1 -> SHA1.digest_of_channel

let to_hex s =
  (* not really optimized... *)
  let u = String.create (String.length s * 2) in
  for k = 0 to String.length s - 1 do
    let v = Printf.sprintf "%02x" (Char.code s.[k]) in
    String.blit v 0 u (2*k) 2
  done;
  u
